SystemOrganization addCategory: #'Fuel-Tests'!
SystemOrganization addCategory: #'Fuel-Core'!
SystemOrganization addCategory: #'Fuel-Core-Cluster'!

TestCase subclass: #FLSerializationTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Tests'!

!FLSerializationTest methodsFor: 'tests' stamp: 'MartinDias 9/22/2010 14:28'!
assertSerializationOf: anObject

	self assertSerializationOfAll: {anObject}! !

!FLSerializationTest methodsFor: 'tests' stamp: 'MartinDias 9/22/2010 14:49'!
assertSerializationOfAll: rootsCollection
	| aByteArray aSerializer materializedRoots |

	aSerializer := FLSerializer on: (RWBinaryOrTextStream on: '') binary.
	aByteArray := (aSerializer serializeAll: rootsCollection) stream contents.
	materializedRoots := 
		(FLMaterializer on: aByteArray readStream) 
			materialize;
			materializedRoots..

	self assert: rootsCollection size equals: materializedRoots size.
	self should: [ rootsCollection includesAllOf: materializedRoots ]! !

!FLSerializationTest methodsFor: 'tests' stamp: 'MartinDias 9/22/2010 14:30'!
testOrderedCollection

	self assertSerializationOf: (OrderedCollection with: 10 with: 20)! !

!FLSerializationTest methodsFor: 'tests' stamp: 'MartinDias 9/23/2010 11:44'!
testPair
	self assertSerializationOf:
			(Pair new
				left: 20;
				right: 30;
				yourself)! !

!FLSerializationTest methodsFor: 'tests' stamp: 'MartinDias 9/22/2010 14:30'!
testPoint
	self assertSerializationOf: 10@20! !

!FLSerializationTest methodsFor: 'tests' stamp: 'MartinDias 9/22/2010 14:31'!
testRectangule
	
	self assertSerializationOf: (10@20 corner: 30@40)! !

!FLSerializationTest methodsFor: 'tests' stamp: 'MartinDias 9/21/2010 00:11'!
testSmallInteger

	self assertSerializationOf: 4000.! !

!FLSerializationTest methodsFor: 'tests' stamp: 'TristanBourgois 9/23/2010 08:53'!
testString
	self assertSerializationOf: 'toto'! !

!FLSerializationTest methodsFor: 'tests' stamp: 'MartinDias 9/23/2010 10:49'!
testSymbol
	self assertSerializationOf: #toto! !

!FLSerializationTest methodsFor: 'tests' stamp: 'TristanBourgois 9/23/2010 10:42'!
testToto
|t|
	t:=Pair new.
	t left:20;right:30.
	self assertSerializationOf: t! !

Object subclass: #FLBenchmarks
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Tests'!

!FLBenchmarks commentStamp: '<historical>' prior: 0!
Usage:

FLBenchmarks new runBenchWith: FLBenchmarks new rectangles 

FLBenchmarks new runBenchWith: FLBenchmarks new points
FLBenchmarks new runBenchWith: FLBenchmarks new plainPairs

FLBenchmarks new runBenchWith: FLBenchmarks new nestedPairs

FLBenchmarks new runBenchWith: FLBenchmarks new plainSmallOrderedCollections 
FLBenchmarks new runBenchWith: FLBenchmarks new orderedCollectionsWithSharedElement 

FLBenchmarks new runBenchWith: FLBenchmarks new strings
FLBenchmarks new runBenchWith: FLBenchmarks new symbols
!

!FLBenchmarks methodsFor: 'serializing' stamp: 'MartinDias 9/20/2010 23:20'!
fuelSerializationBlock
	
	^ [ :aCollection | 
		| serializer |
		serializer := FLSerializer on: (RWBinaryOrTextStream on: '') binary.
		serializer serializeAll: aCollection.
		serializer stream contents ]! !

!FLBenchmarks methodsFor: 'test set' stamp: 'MartinDias 9/20/2010 23:40'!
nestedPairs 
	| aSet numberOfLeafs |
	numberOfLeafs := 2 raisedTo: 14.
	aSet := OrderedCollection new.
	1 to: numberOfLeafs do: [ :i | 
		aSet add: (
			Pair new 
				left: (2 * i); 
				right: (2 * i) + 1) ].
	
	[aSet size > 1] 
		whileTrue: [
			aSet := aSet pairsCollect: [ :leftPair :rightPair | 
				Pair new 
					left: leftPair;
					right: rightPair;
					yourself ] ].
	^aSet! !

!FLBenchmarks methodsFor: 'test set' stamp: 'MartinDias 9/22/2010 16:34'!
orderedCollectionsWithSharedElement
	| aSet aCollection |
	aSet := OrderedCollection new.
	1 to: (2 raisedTo: 9) do: [ :i | 
		aCollection := OrderedCollection new.
		100 timesRepeat: [ aCollection add: 17 ].
		aSet add: aCollection].
	^aSet.! !

!FLBenchmarks methodsFor: 'test set' stamp: 'MartinDias 9/20/2010 23:40'!
plainPairs
	| aSet |
	aSet := OrderedCollection new.
	1 to: (2 raisedTo: 15) by: 2 do: [ :i | 
		aSet add: (Pair new left: i; right: i+1)].
	^aSet.! !

!FLBenchmarks methodsFor: 'test set' stamp: 'MartinDias 9/22/2010 16:31'!
plainSmallOrderedCollections
	| aSet |
	aSet := OrderedCollection new.
	1 to: (2 raisedTo: 14) by: 2 do: [ :i | 
		aSet add: (OrderedCollection with: i with: i+1)].
	^aSet.! !

!FLBenchmarks methodsFor: 'test set' stamp: 'MartinDias 9/20/2010 23:40'!
points
	| aSet |
	aSet := OrderedCollection new.
	1 to: (2 raisedTo: 15) by: 2 do: [ :i | aSet add: i@(i+1)].
	^aSet.! !

!FLBenchmarks methodsFor: 'test set' stamp: 'MartinDias 9/20/2010 23:40'!
rectangles
	| rectangles |
	rectangles := OrderedCollection new.
	1 to: (2 raisedTo: 15) by: 4 do: [ :i | 
		rectangles add: (i@(i+1) corner: (i+2)@(i+3))].
	^rectangles.! !

!FLBenchmarks methodsFor: 'serializing' stamp: 'MartinDias 9/21/2010 11:27'!
referenceStreamSerializationBlock
	^ [ :aCollection | 
		| stream |
		stream := ReferenceStream on: (RWBinaryOrTextStream on: '') binary.
		stream nextPutAll: aCollection.
		stream contents]! !

!FLBenchmarks methodsFor: 'testing' stamp: 'MartinDias 9/22/2010 13:37'!
runBenchWith: aCollection
	Transcript clear.

	self runFuelBenchWith: aCollection.
	self runReferenceStreamBenchWith: aCollection.
	"self runMC2BenchWith: aCollection."

	Transcript flush.
! !

!FLBenchmarks methodsFor: 'testing' stamp: 'MartinDias 9/21/2010 12:53'!
runFuelBenchWith: aCollection

	| aByteArray materializer |

	Transcript show: '= Fuel Serialization ='; cr.
	self spy: [ aByteArray := self fuelSerializationBlock value: aCollection ] putResultsOn: Transcript.

	Transcript show: '= Fuel Materialization ='; cr.
	materializer := FLMaterializer on: aByteArray readStream.
	self spy: [ materializer materialize ] putResultsOn: Transcript.

! !

!FLBenchmarks methodsFor: 'serializing' stamp: 'MartinDias 9/22/2010 13:37'!
runMC2BenchWith: aCollection
"
	| serializer writeStream |

	writeStream := (RWBinaryOrTextStream on: '') binary.

	Transcript show: '= MC2 Serialization ='; cr.
	self spy: [
		serializer := MDSerializer on: writeStream. 
		serializer serialize: aCollection ]
			putResultsOn: Transcript.

	Transcript show: '= MC2 Materialization ='; cr.
	self spy: [ 
		MDMaterializer materialize: writeStream contents ] 
			putResultsOn: Transcript."! !

!FLBenchmarks methodsFor: 'testing' stamp: 'MartinDias 9/21/2010 12:53'!
runReferenceStreamBenchWith: aCollection

	| aByteArray stream referenceStreamMaterializedObjects |

	Transcript show: '= ReferenceStream Serialization ='; cr.
	self spy: [ aByteArray := self referenceStreamSerializationBlock value: aCollection ] putResultsOn: Transcript.

	Transcript show: '= ReferenceStream Materialization ='; cr.
	stream := ReferenceStream on: aByteArray readStream.
	referenceStreamMaterializedObjects := OrderedCollection new.
	self spy: [
		[ stream atEnd ] 
			whileFalse: [
				"referenceStreamMaterializedObjects add: "stream next ]
	] putResultsOn: Transcript.
! !

!FLBenchmarks methodsFor: 'testing' stamp: 'MartinDias 9/20/2010 19:29'!
spy: aBlockClosure putResultsOn: aWriteStream

	| messageTally |
	messageTally := MessageTally new 
		spyEvery: 1 on: aBlockClosure;
		yourself.
	
	aWriteStream nextPutAll: 'tally: ', messageTally tally printString, ' time: ', messageTally time printString; cr. ! !

!FLBenchmarks methodsFor: 'test set' stamp: 'MartinDias 9/23/2010 11:47'!
strings
	| anOrderedCollection |
	anOrderedCollection := OrderedCollection new.
	1 to: (2 raisedTo: 13) by: 2 do: [ :i | anOrderedCollection add: (Titi with: 'toto' with:'titi'  with:i) ].
	^anOrderedCollection.! !

!FLBenchmarks methodsFor: 'test set' stamp: 'MartinDias 9/23/2010 11:55'!
symbols
	| anOrderedCollection allSymbols |
	anOrderedCollection := OrderedCollection new.
	allSymbols := Symbol allSymbols.
	1 to: (2 raisedTo: 11) do: [ :index | 
		anOrderedCollection add: (allSymbols at: index) ].
	^ anOrderedCollection! !

Object subclass: #FLClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLClusterSerializer methodsFor: 'as yet unclassified' stamp: 'sd 9/8/2010 14:35'!
id
	"Answer an unique byte identifier"
	self subclassResponsibility ! !

!FLClusterSerializer methodsFor: 'as yet unclassified' stamp: 'MartinDias 9/11/2010 21:31'!
materializeOn: aReadStream addTo: aCollection mainMaterializer: aMaterializer

	self subclassResponsibility ! !

!FLClusterSerializer methodsFor: 'as yet unclassified' stamp: 'MartinDias 9/12/2010 16:18'!
serializeAll: clusterInstances on: aWriteStream mainSerializer: aSerializer

	self subclassResponsibility ! !

FLClusterSerializer subclass: #FLObjectClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLObjectClusterSerializer methodsFor: 'as yet unclassified' stamp: 'sd 9/8/2010 16:39'!
id

	^0! !

!FLObjectClusterSerializer methodsFor: 'as yet unclassified' stamp: 'MartinDias 9/22/2010 16:09'!
materializeOn: aReadStream addTo: aCollection mainMaterializer: aMaterializer
	"TODO make me nice, please!!"

	| instanceCount className class clusterInstances |
	className := aReadStream nextString.
	instanceCount := aReadStream nextWord.

	class := Smalltalk at: className asSymbol.
	clusterInstances := OrderedCollection new: instanceCount.


	class isVariable 
		ifTrue: [
			instanceCount timesRepeat: [ clusterInstances add: (class basicNew: aReadStream nextWord) ].
			aMaterializer referenceMaterializingActions 
				add: [ self materializeVariableReferencesOfAll: clusterInstances on: aReadStream mainMaterializer: aMaterializer ] ]
		ifFalse: [
			instanceCount timesRepeat: [ clusterInstances add: class basicNew ] ].

	class instSize > 0
		ifTrue: [
			aMaterializer referenceMaterializingActions 
				add: [ self materializeReferencesOfAll: clusterInstances on: aReadStream mainMaterializer: aMaterializer ]].

	aCollection addAll: clusterInstances.
! !

!FLObjectClusterSerializer methodsFor: 'as yet unclassified' stamp: 'MartinDias 9/11/2010 19:24'!
materializeReferencesOfAll: instances on: stream mainMaterializer: aSerializer
	| referenceIndex referencedInstance |
	instances
		do: [ :instance |
			1 to: instance class instSize 
				do: [ :index | 
					referenceIndex := stream nextWord.
					referencedInstance := aSerializer materializedInstances at: referenceIndex.
					instance instVarAt: index put: referencedInstance ]]! !

!FLObjectClusterSerializer methodsFor: 'as yet unclassified' stamp: 'MartinDias 9/22/2010 16:08'!
materializeVariableReferencesOfAll: instances on: stream mainMaterializer: aSerializer
	| referenceIndex referencedInstance |
	instances
		do: [ :instance |
			1 to: instance basicSize 
				do: [ :index | 
					referenceIndex := stream nextWord.
					referencedInstance := aSerializer materializedInstances at: referenceIndex.
					instance basicAt: index put: referencedInstance ]]! !

!FLObjectClusterSerializer methodsFor: 'as yet unclassified' stamp: 'MartinDias 9/22/2010 16:09'!
serializeAll: clusterInstances on: aWriteStream mainSerializer: aSerializer
	"TODO make me nice, please!!"

	| class |
	class := clusterInstances anyOne class.

	aWriteStream nextStringPut: class name.
	aWriteStream nextWordPut: clusterInstances size.

	class isVariable 
		ifTrue: [
			clusterInstances do: [ :instance | aWriteStream nextWordPut: instance basicSize].
			
			aSerializer referenceSerializingActions 
				add: [self serializeVariableReferencesOfAll: clusterInstances on: aWriteStream mainSerializer: aSerializer ]].

	class instSize > 0 
		ifTrue: [ 
			aSerializer referenceSerializingActions 
				add: [self serializeReferencesOfAll: clusterInstances on: aWriteStream mainSerializer: aSerializer ]]
			
! !

!FLObjectClusterSerializer methodsFor: 'as yet unclassified' stamp: 'MartinDias 9/11/2010 18:51'!
serializeReferencesOfAll: instances on: stream mainSerializer: aSerializer 
	| class size |
	
	class := instances anyOne class.
	size := class instSize.
	instances 
		do: [ :anInstance |
			1 to: size do: [ :index |
				stream 	nextWordPut: (aSerializer instanceIndexes at: (anInstance instVarAt: index))]]
				
	! !

!FLObjectClusterSerializer methodsFor: 'as yet unclassified' stamp: 'MartinDias 9/22/2010 15:37'!
serializeVariableReferencesOfAll: instances on: aWriteStream mainSerializer: aSerializer

	instances 
		do: [ :anInstance |
			1 to: anInstance basicSize do: [ :index |
				aWriteStream nextWordPut: (aSerializer instanceIndexes at: (anInstance basicAt: index))]]! !

FLClusterSerializer subclass: #FLPrimitiveClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLPrimitiveClusterSerializer methodsFor: 'as yet unclassified' stamp: 'sd 9/8/2010 17:50'!
materializeOn: aReadStream

	self subclassResponsibility ! !

!FLPrimitiveClusterSerializer methodsFor: 'materializing' stamp: 'MartinDias 9/11/2010 21:30'!
materializeOn: aReadStream addTo: aCollection mainMaterializer: aMaterializer 
	
	| instanceCount |
	instanceCount := aReadStream nextWord.

	instanceCount timesRepeat: 
		[aCollection add: 
			(self materializeOn: aReadStream)].! !

!FLPrimitiveClusterSerializer methodsFor: 'as yet unclassified' stamp: 'sd 9/8/2010 22:08'!
serialize: aByte on: aWriteStream
	self subclassResponsibility! !

!FLPrimitiveClusterSerializer methodsFor: 'as yet unclassified' stamp: 'MartinDias 9/11/2010 18:32'!
serializeAll: aCollection on: aWriteStream mainSerializer: anObject

	aWriteStream nextWordPut: aCollection size.
	aCollection do: [:instance| self serialize: instance on: aWriteStream ]! !

FLPrimitiveClusterSerializer subclass: #FLSmallIntegerClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLSmallIntegerClusterSerializer methodsFor: 'as yet unclassified' stamp: 'sd 9/8/2010 14:46'!
id

	^2! !

!FLSmallIntegerClusterSerializer methodsFor: 'as yet unclassified' stamp: 'sd 9/8/2010 17:50'!
materializeOn: aReadStream

	| hi lo |
	hi := aReadStream next.
	lo := aReadStream next.
	^ (hi bitShift: 8) + lo! !

!FLSmallIntegerClusterSerializer methodsFor: 'as yet unclassified' stamp: 'sd 9/8/2010 22:08'!
serialize: aSmallInteger on: aWriteStream

	aWriteStream nextPut: (aSmallInteger bitShift: -8).
	aWriteStream nextPut: (aSmallInteger bitAnd: 255).! !

FLPrimitiveClusterSerializer subclass: #FLSymbolClusterSerializer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core-Cluster'!

!FLSymbolClusterSerializer methodsFor: 'as yet unclassified' stamp: 'MartinDias 9/23/2010 11:00'!
id

	^3! !

!FLSymbolClusterSerializer methodsFor: 'as yet unclassified' stamp: 'MartinDias 9/23/2010 11:08'!
materializeOn: aReadStream

	| size aByteArray  |
	size := aReadStream nextWord.
	aByteArray := aReadStream next: size.
	^ aByteArray asString asSymbol ! !

!FLSymbolClusterSerializer methodsFor: 'as yet unclassified' stamp: 'MartinDias 9/23/2010 10:57'!
serialize: aSymbol on: aWriteStream

	aWriteStream nextWordPut: aSymbol  size.
	aWriteStream nextPutAll: aSymbol asString.! !

Object subclass: #FLMaterializer
	instanceVariableNames: 'stream clusterSerializers referenceMaterializingActions materializedInstances materializedRoots'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core'!

!FLMaterializer class methodsFor: 'creation' stamp: 'MartinDias 9/20/2010 20:34'!
on: aReadStream

	^self new
		stream: aReadStream; 
		yourself
! !

!FLMaterializer methodsFor: 'private' stamp: 'sd 9/8/2010 15:47'!
clusterSerializerFor: id

	^clusterSerializers at: id.! !

!FLMaterializer methodsFor: 'initialize-release' stamp: 'MartinDias 9/23/2010 11:03'!
initialize 
	
	clusterSerializers := IdentityDictionary new.
	self register: FLObjectClusterSerializer new.
	self register: FLSmallIntegerClusterSerializer new.
	self register:  FLSymbolClusterSerializer new.
	referenceMaterializingActions := OrderedCollection new.! !

!FLMaterializer methodsFor: 'materializing' stamp: 'MartinDias 9/22/2010 14:37'!
materialize
	self materializeInstances.
	self materializeReferences.
	self materializeRoots.! !

!FLMaterializer methodsFor: 'private' stamp: 'MartinDias 9/11/2010 19:22'!
materializeCluster
	| clusterSerializerId instanceCount clusterSerializer |
	clusterSerializerId := stream nextWord.
	clusterSerializer := self clusterSerializerFor: clusterSerializerId.
	clusterSerializer materializeOn: stream addTo: materializedInstances mainMaterializer: self! !

!FLMaterializer methodsFor: 'private' stamp: 'MartinDias 9/21/2010 00:33'!
materializeInstances

	| clusterCount objectCount |
	objectCount := stream nextWord.
	materializedInstances := OrderedCollection new: objectCount.

	clusterCount := stream nextWord.
	clusterCount timesRepeat: [self materializeCluster].
! !

!FLMaterializer methodsFor: 'private' stamp: 'MartinDias 9/11/2010 19:08'!
materializeReferences

	referenceMaterializingActions
		do: [ :aBlock | aBlock value ]
! !

!FLMaterializer methodsFor: 'private' stamp: 'MartinDias 9/22/2010 14:50'!
materializeRoots

	| rootsSize |
	
	rootsSize := stream nextWord.
	materializedRoots := OrderedCollection new: rootsSize.
	rootsSize timesRepeat: [
		| rootIndex |
		rootIndex := stream nextWord.
		materializedRoots 
			add: (materializedInstances at: rootIndex) ]! !

!FLMaterializer methodsFor: 'accessing' stamp: 'MartinDias 9/21/2010 00:32'!
materializedInstances
	^ materializedInstances! !

!FLMaterializer methodsFor: 'accessing' stamp: 'MartinDias 9/22/2010 14:50'!
materializedRoots 

	^ materializedRoots! !

!FLMaterializer methodsFor: 'accessing' stamp: 'MartinDias 9/11/2010 19:09'!
referenceMaterializingActions
	^ referenceMaterializingActions! !

!FLMaterializer methodsFor: 'private' stamp: 'sd 9/8/2010 15:55'!
register: aClusterSerializer

	clusterSerializers at: aClusterSerializer id put: aClusterSerializer. ! !

!FLMaterializer methodsFor: 'accessing' stamp: 'sd 9/8/2010 15:23'!
stream: aStream
	stream:= aStream.! !

Object subclass: #FLReferencesAnalyzer
	instanceVariableNames: 'clusteredInstances objectCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core'!

!FLReferencesAnalyzer methodsFor: 'analysis' stamp: 'TristanBourgois 9/23/2010 09:32'!
analyzeAll: aCollection
	aCollection 
		do: [ :anObject | anObject accept: self ]
! !

!FLReferencesAnalyzer methodsFor: 'accessing' stamp: 'MartinDias 9/21/2010 00:05'!
clusteredInstances
	
	^ clusteredInstances! !

!FLReferencesAnalyzer methodsFor: 'visiting' stamp: 'MartinDias 9/21/2010 00:04'!
initialize 

	clusteredInstances:= IdentityDictionary new.
	objectCount := 0.

! !

!FLReferencesAnalyzer methodsFor: 'accessing' stamp: 'MartinDias 9/20/2010 23:57'!
objectCount
	^ objectCount! !

!FLReferencesAnalyzer methodsFor: 'visiting' stamp: 'MartinDias 9/20/2010 23:50'!
visitObject: anObject
	(clusteredInstances 
		at: anObject class 
		ifAbsentPut: [ OrderedCollection new ]) 
			add: anObject.
	objectCount := objectCount + 1.! !

Object subclass: #FLSerializer
	instanceVariableNames: 'stream clusterSerializerDict defaultClusterSerializer instanceIndexes referenceSerializingActions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Core'!

!FLSerializer class methodsFor: 'as yet unclassified' stamp: 'sd 9/8/2010 14:59'!
on: aWriteStream

	| serializer |
	serializer := self new.
	serializer stream: aWriteStream.
	^serializer.! !

!FLSerializer methodsFor: 'private' stamp: 'sd 9/8/2010 15:58'!
clusterSerializerFor: aClass 
	
	^clusterSerializerDict at: aClass ifAbsent: [ defaultClusterSerializer ].! !

!FLSerializer methodsFor: 'initialize-release' stamp: 'MartinDias 9/23/2010 11:01'!
initialize 
	defaultClusterSerializer := FLObjectClusterSerializer new.

	clusterSerializerDict := IdentityDictionary new.
	clusterSerializerDict at: SmallInteger put: FLSmallIntegerClusterSerializer new.
	clusterSerializerDict at: ByteSymbol put: FLSymbolClusterSerializer new.
	instanceIndexes := IdentityDictionary new.

	referenceSerializingActions := OrderedCollection new.! !

!FLSerializer methodsFor: 'accessing' stamp: 'MartinDias 9/11/2010 18:53'!
instanceIndexes
	^ instanceIndexes! !

!FLSerializer methodsFor: 'accessing' stamp: 'MartinDias 9/11/2010 18:53'!
instanceIndexes: anObject
	instanceIndexes := anObject! !

!FLSerializer methodsFor: 'accessing' stamp: 'MartinDias 9/11/2010 19:01'!
referenceSerializingActions
	^referenceSerializingActions! !

!FLSerializer methodsFor: 'serializing' stamp: 'MartinDias 9/22/2010 14:36'!
serializeAll: rootsCollection
	
	self serializeInstances: rootsCollection.
	self serializeReferences.
	self serializeRoots: rootsCollection.
! !

!FLSerializer methodsFor: 'private' stamp: 'TristanBourgois 9/23/2010 10:33'!
serializeInstances: rootsCollection

	| serializer counter analyzer clusteredInstances |
	
	analyzer:= FLReferencesAnalyzer new 
		analyzeAll: rootsCollection;
		yourself.

	clusteredInstances := analyzer clusteredInstances.

	stream nextWordPut: analyzer objectCount.
	stream nextWordPut: clusteredInstances keys size.
	counter := 1.
	clusteredInstances
		keysAndValuesDo: [ :class :instances | 
			serializer := self clusterSerializerFor: class.
			
			stream nextWordPut: serializer id.
			serializer serializeAll: instances on: stream mainSerializer: self.
			instances
				do: [ :instance | 
					instanceIndexes at: instance put: counter.
					counter := counter + 1 ] ]! !

!FLSerializer methodsFor: 'private' stamp: 'MartinDias 9/11/2010 19:04'!
serializeReferences

	referenceSerializingActions 
		do: [ :aBlock | aBlock value ]! !

!FLSerializer methodsFor: 'serializing' stamp: 'MartinDias 9/22/2010 14:35'!
serializeRoots: rootsCollection

	stream nextWordPut: rootsCollection size.
	
	rootsCollection
		do: [ :instance | 
			stream nextWordPut: (instanceIndexes at: instance)]! !

!FLSerializer methodsFor: 'accessing' stamp: 'sd 9/8/2010 14:59'!
stream
	^ stream! !

!FLSerializer methodsFor: 'accessing' stamp: 'sd 9/8/2010 14:59'!
stream: anObject
	stream := anObject! !

!Object methodsFor: '*Fuel-analysis' stamp: 'MartinDias 9/22/2010 16:13'!
accept: aVisitor

	aVisitor visitObject: self.

	self class isVariable 
		ifTrue: [ 
			1 to: self basicSize 
				do: [ :index | (self basicAt: index) accept: aVisitor ] ].
			
	1 to: self class instSize 
		do: [ :index | (self instVarAt: index) accept: aVisitor ]! !

Object subclass: #Pair
	instanceVariableNames: 'left right'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Tests'!

!Pair methodsFor: 'as yet unclassified' stamp: 'TristanBourgois 9/23/2010 10:45'!
=aPair

	(aPair isKindOf:Pair)ifTrue:[
		^(self left=aPair left)and:[self right=aPair right]] .
	^false! !

!Pair methodsFor: 'accessing' stamp: 'MartinDias 9/16/2010 19:58'!
left
	^ left! !

!Pair methodsFor: 'accessing' stamp: 'MartinDias 9/16/2010 19:58'!
left: anObject
	left := anObject! !

!Pair methodsFor: 'accessing' stamp: 'MartinDias 9/16/2010 19:58'!
right
	^ right! !

!Pair methodsFor: 'accessing' stamp: 'MartinDias 9/16/2010 19:58'!
right: anObject
	right := anObject! !

Object subclass: #Titi
	instanceVariableNames: 'name surname age'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Fuel-Tests'!

!Titi classSide methodsFor: 'as yet unclassified' stamp: 'TristanBourgois 9/23/2010 09:31'!
with: aName with: aSurname with: anAge
	|titi|
	titi:=Titi new.
	titi name:aName.
	titi surname:aSurname.
	titi age:anAge.
	^titi! !

!Titi methodsFor: 'as yet unclassified' stamp: 'TristanBourgois 9/23/2010 09:00'!
= aTiti

	(aTiti isKindOf: Titi)ifTrue:[
		^ ((aTiti age=self age) and:[aTiti name1=self name1])and:[aTiti surname=self surname].
		].
		^false! !

!Titi methodsFor: 'accessing' stamp: 'TristanBourgois 9/23/2010 08:59'!
age
	^ age! !

!Titi methodsFor: 'accessing' stamp: 'TristanBourgois 9/23/2010 08:59'!
age: anObject
	age := anObject! !

!Titi methodsFor: 'accessing' stamp: 'TristanBourgois 9/23/2010 08:59'!
name1
	^ name! !

!Titi methodsFor: 'accessing' stamp: 'TristanBourgois 9/23/2010 08:59'!
name: anObject
	name := anObject! !

!Titi methodsFor: 'accessing' stamp: 'TristanBourgois 9/23/2010 09:00'!
surname
	^ surname! !

!Titi methodsFor: 'accessing' stamp: 'TristanBourgois 9/23/2010 09:00'!
surname: anObject
	surname := anObject! !

!Titi methodsFor: 'as yet unclassified' stamp: 'TristanBourgois 9/23/2010 09:06'!
with: aName with: aSurname with: anAge

	name:=aName.
	surname:=aSurname.
	age:=anAge.! !
